@@ -1,5 +1,7 @@
+from collections import defaultdict
+from datetime import date
 from enum import Enum
-from functools import partial
+from functools import cached_property, partial
 from typing import Any, List, Optional, Type, TypeVar, Union
 import forge
 from django.db import models
@@ -14,18 +16,13 @@ from .base import TBaseModel, TCreateModel, TUpdateModel
 from . import BaseRouter, RouterSchema, Method
 
 
-def _partial_with_description(func, /, *args, **kwargs):
-    wrapped = partial(func, *args, **kwargs)
-    wrapped.__doc__ = func.__doc__
-    return wrapped
-
-
 TDjangoModel = TypeVar('TDjangoModel', bound=models.Model)
 
 
 class DjangoRouterSchema(RouterSchema):
     __router = None
 
+    parent: Optional['DjangoRouterSchema'] = None
     model: Type[TDjangoModel]
     get: Type[TBaseModel]
     create: Type[TCreateModel] = None
@@ -35,22 +32,6 @@ class DjangoRouterSchema(RouterSchema):
     aggregate_fields: Optional[Type[Enum]] = None
     aggregate_group_by: Optional[Type[Enum]] = None
 
-    @property
-    def name_singular(self) -> str:
-        self.name_singular = _name = self.model.__name__.lower()
-        return _name
-
-    @property
-    def related_name_on_parent(self) -> str:
-        self.related_name_on_parent = _name = self.name
-        return _name
-
-    def get_queryset(self, parent_ids: Optional[List[str]] = None, access: Optional[Access] = None):
-        if self.parent:
-            return getattr(self.parent.get_queryset(parent_ids[:-1], access).get(pk=parent_ids[-1]), self.related_name_on_parent).filter(self.objects_filter(access))
-
-        return self.model.objects.filter(self.objects_filter(access))
-
     def __init__(self, **data: Any) -> None:
         super().__init__(**data)
 
@@ -58,14 +39,127 @@ class DjangoRouterSchema(RouterSchema):
             # create_multi is WIP
             raise NotImplementedError("create_multi is not supported for DjangoRouterSchema")
 
+    @property
+    def name_singular(self) -> str:
+        return self.model.__name__.lower()
+
+    @property
+    def related_name_on_parent(self) -> str:
+        return self.name
+
+    @property
+    def id_field(self):
+        name = self.name_singular
+        if self.parent:
+            name = name.removeprefix(self.parent.name_singular)
+
+        return '%s_id' % name
+
+    @property
+    def id_field_placeholder(self):
+        return '/{%s}' % self.id_field
+
+    @cached_property
+    def model_fields(self):
+        def _get_model_fields(model, prefix=''):
+            fields = []
+            for field in model._meta.get_fields():
+                if isinstance(field, (models.ManyToManyRel, models.ManyToOneRel)):
+                    fields.append(f'{prefix}{field.name}__count')
+                    continue
+
+                if isinstance(field, models.ForeignKey):
+                    if self.parent and field.related_model == self.parent.model:
+                        continue
+
+                    fields += _get_model_fields(field.related_model, prefix=prefix + field.name + '__')
+                    continue
+
+                fields.append(f'{prefix}{field.name}')
+
+            return fields
+
+        return Enum(f'{self.model.__name__}Fields', {field: field for field in _get_model_fields(self.model)})
+
+    @property
+    def router(self) -> APIRouter:
+        if not self.__router:
+            self._create_router()
+
+        return self.__router
+
+    def _create_router(self):
+        prefix = f'/{self.name}'
+        if self.parent:
+            prefix = self.parent.id_field_placeholder + prefix
+
+        self.__router = APIRouter(prefix=prefix)
+
+        if self.list and self.list is not ...:
+            self._create_route_list()
+
+        if self.aggregate_fields:
+            self._create_route_aggregate()
+
+        if self.create and self.create is not ...:
+            self._create_route_post()
+
+        if self.get and self.get is not ...:
+            self._create_route_get()
+
+        if self.update and self.update is not ...:
+            self._create_route_patch()
+            self._create_route_put()
+
+        if self.delete:
+            self._create_route_delete()
+
+        child: DjangoRouterSchema
+        for child in self.children:
+            self.__router.include_router(child.router)
+
+    def _create_route_list(self):
+        self.__router.add_api_route('', methods=['GET'], endpoint=self._create_endpoint_list(), response_model=self.list)
+
+    def _create_route_aggregate(self):
+        self.__router.add_api_route('/aggregate/{aggregation_function}/{field}', methods=['GET'], endpoint=self._create_endpoint_aggregate())  # TODO response_model
+
+    def _create_route_post(self):
+        self.__router.add_api_route('', methods=['POST'], endpoint=self._create_endpoint_post(), response_model=Union[self.get, List[self.get]] if self.create_multi else self.get)
+
+    def _create_route_get(self):
+        self.__router.add_api_route(self.id_field_placeholder, methods=['GET'], endpoint=self._create_endpoint_get(), response_model=self.get)
+
+    def _create_route_patch(self):
+        self.__router.add_api_route(self.id_field_placeholder, methods=['PATCH'], endpoint=self._create_endpoint_patch(), response_model=self.get)
+
+    def _create_route_put(self):
+        self.__router.add_api_route(self.id_field_placeholder, methods=['PUT'], endpoint=self._create_endpoint_put(), response_model=self.get)
+
+    def _create_route_delete(self):
+        self.__router.add_api_route(self.id_field_placeholder, methods=['DELETE'], endpoint=self._create_endpoint_delete(), status_code=204)
+
+    def get_queryset(self, parent_ids: Optional[List[str]] = None, access: Optional[Access] = None):
+        if self.parent:
+            return getattr(self.parent.get_queryset(parent_ids[:-1], access).get(pk=parent_ids[-1]), self.related_name_on_parent).filter(self.objects_filter(access))
+
+        return self.model.objects.filter(self.objects_filter(access)).annotate(models.Count(field.name) for field in self.model._meta.get_fields() if isinstance(field, (models.ManyToManyRel, models.ManyToOneRel)))
+
     def objects_filter(self, access: Optional[Access] = None) -> models.Q:
         if hasattr(self.model, 'tenant_id'):
             return models.Q(tenant_id=access.tenant_id)
 
         return models.Q()
 
-    def objects_get_filtered(self, *, parent_ids: Optional[List[str]] = None, access: Optional[Access] = None, pagination: Pagination) -> List[TDjangoModel]:
-        return list(pagination.query(self.get_queryset(parent_ids, access)))
+    def objects_get_filtered(
+        self,
+        *,
+        parent_ids: Optional[List[str]] = None,
+        access: Optional[Access] = None,
+        search: models.Q = models.Q(),
+        pagination: Pagination,
+    ) -> List[TDjangoModel]:
+        return list(pagination.query(self.get_queryset(parent_ids, access).filter(search)))
 
     def object_get_by_id(self, id: str, parent_ids: Optional[List[str]] = None, access: Optional[Access] = None) -> TDjangoModel:
         return self.get_queryset(parent_ids, access).get(id=id)
@@ -96,12 +190,21 @@ class DjangoRouterSchema(RouterSchema):
         else:
             instance.delete()
 
+    def _get_security_scopes(self, method: Method):
+        if self.security_scopes and self.security_scopes.get(method):
+            return self.security_scopes.get(method)
+
+        if self.parent:
+            return self.parent._get_security_scopes(method)
+
+        return None
+
     def _security_signature(self, method: Method):
         if not self.security:
             return []
 
         return [
-            forge.kwarg('access', type=Access, default=Security(self.security, scopes=self.security_scopes and self.security_scopes.get(method))),
+            forge.kwarg('access', type=Access, default=Security(self.security, scopes=self._get_security_scopes(method))),
         ]
 
     def _path_signature_id(self, include_self=True):
@@ -118,22 +221,107 @@ class DjangoRouterSchema(RouterSchema):
     def _get_id(self, kwargs: dict):
         return kwargs[self._path_signature_id()[-1].name]
 
-    def endpoint_list(self, *, access: Optional[Access] = None, pagination: Pagination, **kwargs):
+    def endpoint_list(self, *, access: Optional[Access] = None, pagination: Pagination, search: models.Q = models.Q(), **kwargs):
         ids = self._get_ids(kwargs, include_self=False)
         return self.list(items=[
             self.get.from_orm(obj)
             for obj in self.objects_get_filtered(
                 parent_ids=ids,
                 access=access,
+                search=search,
                 pagination=pagination,
             )
         ])
 
+    def search_filter(self, **kwargs) -> models.Q:
+        return models.Q(**kwargs)
+
+    def search_filter_fields(self):
+        fields = defaultdict(list)
+        for field in self.model._meta.get_fields():
+            if getattr(field, 'primary_key', False) or field.name == 'tenant_id':
+                continue
+
+            field_type = self.model.__annotations__.get(field.name)
+            field_name = field.name
+
+            if isinstance(field, models.ForeignKey):
+                field_type = str
+                field_name += '__id'
+
+                if self.parent and field.related_model == self.parent.model:
+                    continue
+            
+            if isinstance(field, (models.ManyToManyRel, models.ManyToOneRel)):
+                continue
+
+            is_exact_search_included = True
+            query_options = {
+                'default': None,
+            }
+
+            if isinstance(field, (
+                models.DateField,
+                models.DateTimeField,
+                models.IntegerField,
+                models.DecimalField,
+            )):
+                variations = [(field_name, field_type)]
+
+                if isinstance(field, models.DateTimeField):
+                    variations.append((f'{field_name}__date', date))
+
+                if isinstance(field, (models.DateField, models.DateTimeField)):
+                    for variation, type_ in [*variations]:
+                        if type_ != date:
+                            continue
+
+                        variations.append((f'{variation}__year', int))
+                        variations.append((f'{variation}__quarter', int))
+                        variations.append((f'{variation}__month', int))
+                        variations.append((f'{variation}__day', int))
+                        variations.append((f'{variation}__week', int))
+                        variations.append((f'{variation}__week_day', int))
+
+                for variation in variations:
+                    name = variation
+                    type_ = field_type
+                    if isinstance(variation, tuple):
+                        name, type_ = variation
+
+                    fields[field].append(forge.kwarg(f'{name}__gte', type=Optional[type_], default=Query(**query_options)))
+                    fields[field].append(forge.kwarg(f'{name}__lte', type=Optional[type_], default=Query(**query_options)))
+
+            elif isinstance(field, models.CharField):
+                if field.choices:
+                    fields[field].append(forge.kwarg(f'{field_name}__in', type=Optional[List[field_type]], default=Query(**query_options)))
+                    is_exact_search_included = False
+
+                else:
+                    query_options['max_length'] = field.max_length
+                    fields[field].append(forge.kwarg(f'{field_name}__icontains', type=Optional[field_type], default=Query(**query_options)))
+
+            if is_exact_search_included:
+                fields[field].insert(0, forge.kwarg(field_name, type=Optional[field_type], default=Query(**query_options)))
+
+        return [x for xs in fields.values() for x in xs]
+
+    def create_depends_search(self):
+        return forge.sign(*self.search_filter_fields())(self.search_filter)
+
+    def _depends_search(self):
+        return [
+            forge.kwarg('search', type=models.Q, default=Depends(self.create_depends_search())),
+            forge.kwarg('pagination', type=Pagination, default=Depends(
+                forge.modify('order_by', type=Optional[List[self.model_fields]], default=Query(self.pagination_options.get('default_order_by', list())))(depends_pagination(**self.pagination_options))
+            )),
+        ]
+
     def _create_endpoint_list(self):
         return forge.sign(*[
             *self._path_signature_id(include_self=False),
             *self._security_signature(Method.GET_LIST),
-            forge.kwarg('pagination', type=Pagination, default=Depends(depends_pagination(**self.pagination_options))),
+            *self._depends_search(),
         ])(self.endpoint_list)
 
     def endpoint_aggregate(
@@ -144,14 +332,14 @@ class DjangoRouterSchema(RouterSchema):
         access: Optional[Access] = None,
         pagination: Pagination,
         group_by: Optional[List[str]] = None,
+        search: models.Q = models.Q(),
         **kwargs,
     ):
         ids = self._get_ids(kwargs, include_self=False)
-        q_filters = models.Q()
 
         return aggregation(
             self.get_queryset(ids, access),
-            q_filters=q_filters,
+            q_filters=search,
             aggregation_function=aggregation_function,
             field=field,
             group_by=group_by,
@@ -166,7 +354,7 @@ class DjangoRouterSchema(RouterSchema):
             forge.kwarg('aggregation_function', type=AggregationFunction, default=Path(...)),
             forge.kwarg('field', type=self.aggregate_fields, default=Path(...)),
             forge.kwarg('group_by', type=Optional[List[self.aggregate_group_by]], default=Query(None)) if self.aggregate_group_by else None,
-            forge.kwarg('pagination', type=Pagination, default=Depends(depends_pagination(**self.pagination_options))),
+            *self._depends_search(),
         ] if arg])(self.endpoint_aggregate)
 
     def endpoint_post(self, *, data: TCreateModel, access: Optional[Access] = None, **kwargs):
@@ -237,103 +425,5 @@ class DjangoRouterSchema(RouterSchema):
             *self._security_signature(Method.DELETE),
         ])(self.endpoint_delete)
 
-    @property
-    def id_field(self):
-        name = self.name_singular
-        if self.parent:
-            name = name.removeprefix(self.parent.name_singular)
-
-        return '%s_id' % name
-
-    @property
-    def id_field_placeholder(self):
-        return '/{%s}' % self.id_field
-
-    def _create_route_list(self):
-        self.__router.add_api_route('', methods=['GET'], endpoint=self._create_endpoint_list(), response_model=self.list)
-
-    def _create_route_aggregate(self):
-        self.__router.add_api_route('/aggregate/{aggregation_function}/{field}', methods=['GET'], endpoint=self._create_endpoint_aggregate())  # TODO response_model
-
-    def _create_route_post(self):
-        self.__router.add_api_route('', methods=['POST'], endpoint=self._create_endpoint_post(), response_model=Union[self.get, List[self.get]] if self.create_multi else self.get)
-
-    def _create_route_get(self):
-        self.__router.add_api_route(self.id_field_placeholder, methods=['GET'], endpoint=self._create_endpoint_get(), response_model=self.get)
-
-    def _create_route_patch(self):
-        self.__router.add_api_route(self.id_field_placeholder, methods=['PATCH'], endpoint=self._create_endpoint_patch(), response_model=self.get)
-
-    def _create_route_put(self):
-        self.__router.add_api_route(self.id_field_placeholder, methods=['PUT'], endpoint=self._create_endpoint_put(), response_model=self.get)
-
-    def _create_route_delete(self):
-        self.__router.add_api_route(self.id_field_placeholder, methods=['DELETE'], endpoint=self._create_endpoint_delete(), status_code=204)
-
-    def _create_router(self):
-        prefix = f'/{self.name}'
-        if self.parent:
-            prefix = self.parent.id_field_placeholder + prefix
-
-        self.__router = APIRouter(prefix=prefix)
-
-        if self.list and self.list is not ...:
-            self._create_route_list()
-
-        if self.aggregate_fields:
-            self._create_route_aggregate()
-
-        if self.create and self.create is not ...:
-            self._create_route_post()
-
-        if self.get and self.get is not ...:
-            self._create_route_get()
-
-        if self.update and self.update is not ...:
-            self._create_route_patch()
-            self._create_route_put()
-
-        if self.delete:
-            self._create_route_delete()
-
-        child: DjangoRouterSchema
-        for child in self.children:
-            self.__router.include_router(child.router)
-
-    @property
-    def router(self) -> APIRouter:
-        if not self.__router:
-            self._create_router()
-
-        return self.__router
-
-
-class ModelRouter(BaseRouter):
-    __model: models.Model
-    __router: APIRouter
-    __schema: RouterSchema
-    __security: Optional[SecurityBase]
-
-    def __init_subclass__(cls, model: models.Model, router: APIRouter, schema: RouterSchema, security: Optional[SecurityBase] = None) -> None:
-        cls.__model = model
-        cls.__router = router
-        cls.__schema = schema
-        cls.__security = security
-
-        super().__init_subclass__()
-
-        cls.init_routes()
-
-    @classmethod
-    def _add_api_route(cls, path, func, method='GET', response_model=None):
-        cls.__router.add_api_route(
-            path,
-            endpoint=_partial_with_description(func, cls),
-            response_model=response_model,
-            methods=[method],
-            **getattr(func, 'route_options', {}),
-        )
-
-    @classmethod
-    def init_routes(cls):
-        pass
+    class Config:
+        keep_untouched = (cached_property,)
